; Path: hal.com!decwrl!uunet!zephyr.ens.tek.com!uw-beaver!cornell!raman
; From: raman@cs.cornell.edu (T. V. Raman)
; Newsgroups: gnu.emacs.sources
; Subject: bookmarks in emacs:
; Summary: A bookmark utility for emacs:
; Keywords: bookmarks, returning to last read position,
; Date: 18 Feb 92 02:32:02 GMT
; Organization: Cornell Univ. CS Dept, Ithaca NY 14853
; 
; 
; Hi! 
; 
; This is the first time I am posting something to this group, so I hope
; I get it right.  
; 
; I  hacked up a simple bookmark utility for emacs that I find extremely
; useful and thought I would share it with the net.  I am posting the
; file of code and hope that it is completely stand alone from the rest
; of the emacs hacks I have,  but in case something goes wrong, feel
; free to contact me and I will supply any missing functions.  
; 
; Here is the file containing the code:
; If you happen to make any improvements or changes to this file please
; contact me.  
; 
; Thanks,
; 
; --Raman
;
; 
;    T. V. Raman <raman@cs.cornell.edu>Tel: (607)255-7421 R 272-2435
; 		       Office: 5162 Upson Hall,
; Department of Computer Science, Cornell University Ithaca NY 14853-6201
; 		Res: 226 Bryant Avenue Ithaca NY 14850


;;; Copyright (c) 1991 T. V. Raman.  May be redistributed only under the
;;; terms of the GNU Emacs General Public License.

;; LCD Archive Entry:
;; bookmark|T. V. Raman|raman@cs.cornell.edu
;; |Bookmark position in a file so you may return to where you left off.
;; |92-02-18||~/misc/bookmark.el.Z|

;;; BOOKMARK  :
;;; bookmark.el  ;;; author: raman
;;; defines a bookmark function.
;;; A bookmark keeps track of your position in a file so you may return to read
;;; where you left off.  A bookmark is a list of three things: the name, the
;;; pathname of the file being read, and the position of the point within that
;;; file.  Bookmarks are stored in a bookmark file.  Usually you will put this
;;; file in the same directory as the book itself, but it may be anywhere.

;;; A bookmark consists of the following:
;;; file-name. == file containing the marked position.
;;; position of point. 
;;; name of the book mark. == tag to use to find it again.
;;; Function create bookmark appends a book mark
;;; to the book mark file in the directory containing current file.
;;; If called with prefix arg,
;;; it prompts  for a bookmark file in which to place the bookmark.
;;; You can use this in case you do not have write permissions
;;; to the directory containing the book. 
;;; function locate bookmark takes the name of a book directory,
;;; and queries for a  bookmark name to locate.
;;; Again if called with prefix arg,
;;; it first queries for a bookmark file in which to look for the bookmark
;;; It then visits the file in a new buffer,
;;; and places point at  the position marked by bookmark-name
;;; function query-locate-bookmark goes through all the bookmarks in
;;; a .bookmark file,  and interactively selects one of them.
;;; Has the same functionality as locate-bookmark when called with prefix arg
;;; function query-delete-bookmark allows you to clean up the 
;;; .bookmark file from time to time.
;;; By default the file which is cleaned up is the .bookmark file
;;; for the book being read, ie. the .bookmark file
;;; in the directory of the file currently bisited.
;;; This default behaviour can be altered by calling it with prefix arg,
;;; in which case it  prompts for the .bookmark file.
;;; Thus by default all files in a directory will have their bookmarks
;;; placed in the .bookmark file in that directory.
;;; If create-bookmark has been called with a prefix arg, then this file
;;; will be placed relative to the  home directory.

;;; When locating a bookmark,
;;; the program therefore expects the name of a directory,
;;; and then tries to look for the bookmark in the .bookmark file
;;;  in that directory.
;;; unless called with prefix arg,
;;; in which case it expects the name of an explicit bookmark file.
;;; The default behaviour is controlled by three global variables,
;;; and these can be customised by setting them in the .emacs.


;;; HISTORY

;; Created: 
;;; Sun Oct 27 10:16:51 EST 1991

;;; COMMANDS

(provide 'bookmark)
;;; key definitions:

(global-set-key "\M-:" 'create-bookmark)
(global-set-key "\M-\"" 'locate-bookmark)
(global-set-key "\M-*" 'query-locate-bookmark)
(global-set-key "\C-c\C-d" 'insert-date-and-time)
;;; Use this when creating bookmark if you want the date and time.

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; PUBLIC VARIABLES
;;
(defvar *home-dir* "~/"
  "The home directory")

(defvar *bookmark-file* ".bookmark"
  "the name of the default bookmark file for any book")

(defvar *default-book-directory* (expand-file-name "~/books/")
  " The directory containing the  books.")

(defvar *default-book* (expand-file-name "~/books/abelson/"))

;;; *default-book-directory* points to where all the books are located.
;;; Each book is assumed to have its own subdirectory here,
;;; and *default-book* points to one of these.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; PRIVATE  FUNCTIONS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Constructor:
(defmacro make-bookmark(name file pos)
  (list  'list name file pos)
  )
;;; selectors for bookmark-entry:

(defmacro bookmark-entry-name (entry)
  (list 'car entry))

(defmacro bookmark-entry-file-name(entry)
  (list 'car (list 'cdr entry )))

(defmacro bookmark-entry-position(entry)
  (list 'car (list 'cdr  (list 'cdr entry ))))

;;; marks off sexp and copies it to kill ring.
;;; If mark is set, first copies region to kill ring.
(defun bookmark-copy-sexp-as-kill ()
  (if (mark)
      (copy-region-as-kill (point) (mark))  )
  (let
      ((beg (point)))
    (forward-sexp)
    (copy-region-as-kill beg (point))
    (goto-char beg)
    ))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; PUBLIC FUNCTIONS
;;; function to add new bookmarks.

(defun create-bookmark (&optional arg)
  "appends a bookmark entry
to the .bookmark file in the directory of the current file.
If called with prefix arg, prompts for name of an explicit bookmark file.
By default this the .bookmark file in the home directory.
bookmark entries have the following fields at present:
bookmark-name: a name to use when locating the bookmark
bookmark-file-name: The name of the file being marked
bookmark-position: The position of point that is being marked
The function sets the variable enable-recursive-minibuffers so you can use
all of emacs when creating the bookmark.
The function first marks current sexp and copies it as kill
so typing C-y at the minibuffer yanks this sexp as the bookmark-name."
  (interactive "P")
  (bookmark-copy-sexp-as-kill)
  (let* (
	 (enable-recursive-minibuffers t)
	 (file-name (buffer-file-name))
	 (position (point))
	 (bookmark-name (read-minibuffer "Enter book mark name: C-y for current sexp, C-c C-d for date and time  " "\""))
	 (bookmark-entry (make-bookmark
			  bookmark-name
			  file-name
			  position))
	 (directory-name (file-name-directory file-name))
	 (bookmark-file nil)
	 (bookmark-buffer nil)
	 )
    (if   arg
	(setq bookmark-file 
	      (read-file-name
	       (concat "Bookmark file Default: "
		       (prin1-to-string *bookmark-file*))
	       *home-dir*
	       *bookmark-file*
	       nil))
      (setq bookmark-file (concat directory-name *bookmark-file* )))
    (condition-case nil
	(setq bookmark-buffer (find-file-noselect bookmark-file))
      (error  (message "Cannot open bookmark file %s "
		       bookmark-file) nil))
    (save-excursion
      (set-buffer bookmark-buffer)
      (print bookmark-entry (current-buffer))
      (save-buffer)
      (kill-buffer (current-buffer))
      )
    (princ bookmark-entry)
    )
  )


;;; loop through all bookmarks  querying for each.
;;; open book to selected bookmark.
;;; If called interactively, prompt for a bookmark file to search.
(defun query-locate-bookmark (&optional  arg)
  "Interactively show all bookmarks that have been defined
and allow selection."
  (interactive "P")
  (let  (
	 (bookmark-file nil)
	 (bookmark-name nil)
	 (bookmark-buffer nil)
	 )
    (if arg
    	(setq bookmark-file 
	      (read-file-name
	       (concat "Bookmark file Default: "
		       (prin1-to-string *bookmark-file*))
	       *home-dir*
	       *bookmark-file*
	       t))
      (setq bookmark-file
	    (concat 
	     (read-file-name
	      (concat "Directory containing book: Default "
		      (prin1-to-string *default-book*))
	      *default-book-directory*
	      *default-book*
	      t)
	     *bookmark-file*)
	    ))
    (condition-case nil
	(setq bookmark-buffer (find-file-noselect bookmark-file))
      (error (message "Cannot open bookmark file %s" bookmark-file)
	     nil))
    (setq bookmark-entry (query-and-get-entry bookmark-buffer))
    (kill-buffer bookmark-buffer)
    (if bookmark-entry
	(open-to-bookmark bookmark-entry)
      (message "No bookmark selected")
      )
    )
  )

;;; interactively delete bookmarks.
;;; Works on the bookmark file for book being read.
;;; If called with prefix arg, first prompts for bookmark file.

(defun query-delete-bookmarks (&optional arg)
  (interactive "P")
  (let* (
	 (file-name (buffer-file-name))
	 (directory-name (file-name-directory file-name))
	 (bookmark-file nil)
	 (bookmark-buffer nil)
	 (current-entry nil)
	 )
    (if arg
	(setq bookmark-file 
	      (read-file-name
	       (concat "Bookmark file Default: "
		       (prin1-to-string *bookmark-file*))
	       *home-dir*
	       *bookmark-file*
	       t))
      (setq bookmark-file (concat directory-name *bookmark-file*))
      )
    (condition-case nil
	(setq bookmark-buffer (find-file-noselect bookmark-file))
      (error (message "cannot open bookmark file %s"
		      bookmark-file) nil))
    (setq current-entry (pop-next-entry bookmark-buffer))
    (while
	(not (null current-entry))
      (if (y-or-n-p
	   (concat "delete: "
		   (prin1-to-string
		    (bookmark-entry-name current-entry))))
	  (bookmark-delete-current-entry bookmark-buffer))
      (setq current-entry (pop-next-entry bookmark-buffer))
      )
    (princ "finished purging bookmarks")
    (save-excursion
      (set-buffer bookmark-buffer)
      (save-buffer)
      (kill-buffer bookmark-buffer)
      )
    )
  )

;;; auxilliary functions:
(defun insert-date-and-time()
  (interactive)
  (shell-command "date +DATE:%t%a%t%h%t%d%tTIME:%T " t)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; HELPER FUNCTIONS

;;; find a bookmark.

(defun locate-bookmark (&optional arg)
  "locate bookmark and visit that file
If called with prefix arg,
also prompts for a bookmark file in which to search."
  (interactive "P")
  (let(
       (bookmark-file nil)
       (bookmark-buffer nil)
       (bookmark-name nil)
       )
    (if arg
	(setq bookmark-file 
	      (read-file-name
	       (concat "Bookmark file Default: "
		       (prin1-to-string *bookmark-file*))
	       *home-dir*
	       *bookmark-file*
	       t))
      (setq bookmark-file (concat 
			   (read-file-name
			    (concat "Directory containing Book: Default "
				    (prin1-to-string *default-book*))
			    *default-book-directory*
			    *default-book* 
			    t)
			   *bookmark-file*)
	    ))
    (setq bookmark-name (read-minibuffer "Enter bookmark name: " "\""))
    (condition-case nil
	(setq bookmark-buffer (find-file-noselect bookmark-file))
      (error  (message "Cannot open bookmark file %s" bookmark-file)
	      nil))
    (setq bookmark-entry
	  (locate bookmark-name
		  bookmark-buffer))
    (kill-buffer bookmark-buffer)
    (if  bookmark-entry
	(open-to-bookmark bookmark-entry)
      )
    )
  )

;;; pop next entry from buffer buff. return nil at end.

(defun pop-next-entry( bookmark-buffer)
  (condition-case nil
      (read bookmark-buffer)
    (error nil)
    )
  )

;;; recursively search through bookmark-buffer for bookmark-name.
;;; return corresponding bookmark-entry if found, else nil.
;;; This may be made iterative later.

(defun locate (bookmark-name bookmark-buffer)
  (let
      ((current-entry (pop-next-entry bookmark-buffer)))
    (cond
     ((equal bookmark-name
	     (bookmark-entry-name current-entry)) current-entry)
     ((null current-entry)
      (message "selected bookmark %s not found" bookmark-name)  nil)
     (t (locate bookmark-name bookmark-buffer))
     )
    )
  )


;;; open book to specified position as given by bookmark-entry

(defun open-to-bookmark(bookmark-entry)
  (cond
   ((null bookmark-entry)  nil)
   (t
    (find-file
     (bookmark-entry-file-name  bookmark-entry)) ;;; open the book
    (goto-char
     (bookmark-entry-position bookmark-entry)) ;;; to the specified location
    (recenter 0) ;;; move line to top.
    (setq buffer-read-only t) ;; read only
  ;; Force mode-line redisplay
  (set-buffer-modified-p (buffer-modified-p))
    )
   )
  )

;;; Loop through all entries in bookmark-buffer,
;;; and return the one picked.
;;; return nil if none selected.

(defun query-and-get-entry(bookmark-buffer)
  (let
      ((current-entry (pop-next-entry bookmark-buffer )))
    (cond
     ((null current-entry) (princ "No more bookmarks")nil)
     (t
      (if (y-or-n-p
	   (prin1-to-string (bookmark-entry-name current-entry))
	   )
	  current-entry
	(query-and-get-entry bookmark-buffer)
	)
      )
     )
    )
  )

;;; delete current bookmark  entry  from buffer.
;;; current entry is the one immediately before point, ie. the one just read.

(defun bookmark-delete-current-entry( buffer)
  (save-excursion
    (set-buffer buffer)
    (let ((start (point)))
      (backward-sexp)
      (delete-region start (point))
      ))
  )
